#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\editfile.h> 
#include <owl\dc.h> 
#include "pr63_1.rc"
 
//  .
class TApp: public TApplication
{
  public:
	 TApp():TApplication() {}
	 void InitMainWindow();
};

//  -.
class TCWndw : public TWindow
{
  public:
          TCWndw (TWindow *parent, const char far *title);
  protected:
          BOOL bitmapInClip;
          int lButton, rButton, paste;
          TPen *newPen;
          TClientDC *lineDC, *rectDC;
          TRect rect;

          void EvSetFocus (HWND);
          void EvLButtonDown (UINT, TPoint &point);
          void EvLButtonUp (UINT, TPoint&);
          void EvMouseMove (UINT, TPoint &point);
          void EvRButtonDown (UINT, TPoint &point);
          void EvRButtonUp (UINT, TPoint&);
          void CmEditPaste();
  private:
          void PasteBitmap (TPoint point);
          void CmEnableEditPaste (TCommandEnabler &commandEnabler);

	  DECLARE_RESPONSE_TABLE (TCWndw);
};

DEFINE_RESPONSE_TABLE1 (TCWndw, TWindow)
          EV_WM_SETFOCUS, 
          EV_WM_LBUTTONDOWN, 
          EV_WM_LBUTTONUP, 
          EV_WM_RBUTTONDOWN, 
          EV_WM_RBUTTONUP,
          EV_WM_MOUSEMOVE,
          EV_COMMAND (CM_EDITPASTE, CmEditPaste), 
          EV_COMMAND_ENABLE (CM_EDITPASTE, CmEnableEditPaste), 
END_RESPONSE_TABLE;

//***********************************
//   TCWndw.
//***********************************
// TCWndw::TCWndw()
//   -.
TCWndw::TCWndw (TWindow *parent, const char far *title) :
         TWindow (parent, title) 
{
  //  .
  lButton = FALSE;
  rButton = FALSE;
  paste = FALSE;
  rect.SetEmpty();
}

// TCWndw::EvSetFocus()
//   ,  - 
//    .
void TCWndw::EvSetFocus (HWND)
{
  // ,       .
  if (IsClipboardFormatAvailable (CF_BITMAP))
    //      
    // ,    Paste.
    bitmapInClip = TRUE; 
  else
    //    ,
    //    Paste.
    bitmapInClip = FALSE;
  //     WM_SETFOCUS.
  DefaultProcessing();
}

// TCWndw::EvLButtonDown()
//           WM_LBUTTONDOWN. 
void  TCWndw::EvLButtonDown (UINT, TPoint &point) 
{
  //             ""...
  if (paste) PasteBitmap (point);
  //       ... 
  else if (!lButton)
       { 
         //     . 
         lineDC = new TClientDC (HWindow); 
         newPen = new TPen (TColor::Black, 2, PS_SOLID); 
         lineDC->SelectObject (*newPen);
         //       . 
         SetCapture ();
         //      . 
         lineDC->MoveTo (point);
         //    . 
         lButton = TRUE;
       }
}

// TCWndw::EvLButtonUp()
//      WM_LBUTTONUP. 
void TCWndw::EvLButtonUp (UINT, TPoint&)
{
  if (lButton)
  {
    //   . 
    delete lineDC;
    //    . 
    delete newPen;
    //   . 
    lButton = FALSE;
    //     . 
    ReleaseCapture();
  }
}

// TCWndw::EvMouseMove()
//      WM_MOUSEHOVE.
void TCWndw::EvMouseMove (UINT, TPoint &point)
{
  //    ,  .
  if (lButton) lineDC->LineTo (point);
  //    ...
  else if (rButton)
       {
         //     .
         rectDC->SetROP2 (R2_XORPEN);
         //   . 
         rectDC->DrawFocusRect (rect);
         //      .
         rect.Set (rect.left, rect.top, point.x, point.y);
         //   . 
         rectDC->DrawFocusRect (rect);
       }
}

// TCWndw::EvRButtonDown()
//      WM_RBUTTONDOWN.
void TCWndw::EvRButtonDown (UINT, TPoint &point) 
{
  if (!rButton) 
  {
    //    . 
    rButton = TRUE;
    //     
    //      .
    rectDC = new TClientDC (HWindow);
    rect.Set (point.x, point.y, point.x, point.y);
    //       . 
    SetCapture ();
  }
}

// TCWndw::EvRButtonUp()
//      WM_RBUTTONUP.
void TCWndw::EvRButtonUp (UINT, TPoint&) 
{
  //    .
  rButton = FALSE;
  //   . 
  rectDC->SetROP2 (R2_XORPEN); 
  rectDC->DrawFocusRect (rect);
  //      . 
  delete rectDC; 
  ReleaseCapture();
  //          . 
  int width = rect.right - rect.left; 
  int height = rect.bottom - rect.top;
  //     - ...
  if   ((width > 0) && (height > 0))
  {
    //  DC ( )    .
    TClientDC clientDC (HWindow);
    TMemoryDC memoryDC (clientDC);
    //   .
    TBitmap bitmap(clientDC, width, height, FALSE);
    //     DC . 
    memoryDC.SelectObject (bitmap);
    //     DC . 
    memoryDC.BitBlt (0, 0, width, height, clientDC, rect.left, 
       rect.top, SRCCOPY);
    //     . 
    TClipboard &clipboard = OpenClipboard(); 
    clipboard.EmptyClipboard();
    //      . 
    bitmap.ToClipboard (clipboard);
    //    Clipboard. 
    clipboard.CloseClipboard();
    //  ,    . 
    MessageBox ("    ", "", MB_OK);
    //   Paste . 
    bitmapInClip = TRUE;
  }
}

// TCWndw::CmEditPaste()
//   ,  
//   Paste   Edit.
void TCWndw::CmEditPaste() 
{
  //      .
  paste = TRUE;
  HCURSOR hCursor = LoadCursor(NULL, IDC_CROSS);
  SetClassWord (GCW_HCURSOR, (WORD) hCursor);
}

// TCWndw::PasteBitmap()
//      
//    .
void TCWndw::PasteBitmap(TPoint point) 
{
  //    .
  TClientDC clientDC (HWindow);
  //   .
  TClipboard &clipboard = OpenClipboard();
  //   .
  TBitmap bitmap(clipboard);
  //    Clipboard. 
  clipboard.CloseClipboard();
  //      
  //     .
  TMemoryDC memoryDC (clientDC);
  memoryDC.SelectObject (bitmap);
  //      .
  int width = bitmap.Width();
  int height = bitmap.Height();
  //     . 
  clientDC.BitBlt (point.x, point.y, width, height, 
            memoryDC, 0, 0, SRCCOPY);
  //   . 
  paste = FALSE;
  //     . 
  HCURSOR hCursor = LoadCursor(NULL,IDC_ARROW); 
  SetClassWord (GCW_HCURSOR, (WORD) hCursor);
}

// TCWndw::CmEnableEditPaste()
//     
//  Edit/Paste  ,  
//   .
void TCWndw::CmEnableEditPaste (TCommandEnabler &commandEnabler)
{
  commandEnabler.Enable (bitmapInClip);
}

//***********************************
//   TApp.
//***********************************
// TApp::InitMainWindow()
//      .
void TApp::InitMainWindow()
{
  //  -.
  TWindow *client = new TCWndw(0, 0);
  //   .
  TFrameWindow *wndw = new TFrameWindow 
     (0, "  - 2", client);
  //     . 
  wndw->AssignMenu (MENU_1);
  //     .
  wndw->Attr.X = 40;
  wndw->Attr.Y = 40;
  wndw->Attr.W = GetSystemMetrics (SM_CXSCREEN) / 1.5;
  wndw->Attr.H = GetSystemMetrics (SM_CYSCREEN) / 1.5;
  //   . 
  SetMainWindow(wndw);
} 

int OwlMain(int,char *[])
{
  return TApp().Run();
}
